/*
 * Copyright 2009 Jonathan Rubin
 * This file is part of Camus.
 *
 * Camus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Camus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Camus.  If not, see <http://www.gnu.org/licenses/>.
 */

package cbr.framework;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;

/**
 * <p>CAMUS: Case Assembly, Maintenance & Usage System</p>
 * <p>Bugs, improvements, concerns can be reported to jrub001@aucklanduni.ac.nz<p>
 * 
 * <p>The <code>CaseBase</code> class holds a collection of <code>Cases</code> in a list structure. It provides methods
 *    to acquire, maintain and retrieve cases in the case base.</p>
 *    
 * @author Jonathan Rubin	
 * @version 1.0
 */
public class CaseBase {
	/**
	 * The number of attributes each feature must specify in flat file format.
	 */
	public static final int NUMBER_DETAILS = 3;
	
	private List<Case> caseBase;
	
	/**
	 * Creates an empty case base
	 */
	public CaseBase(){
		caseBase = new ArrayList<Case>();
	}
	
	/**
	 * Creates a case base described in flat file format.
	 * @param fileName the name of the file that holds the cases
	 */
	public CaseBase(final String fileName){
		caseBase = new ArrayList<Case>();
		assembleCaseBase(fileName);
	}
	
	/**
	 * Given a case base represented as a flat file, this method will load the cases into working memory.
	 * @param fileName the name of the file that holds the cases
	 */
	public void assembleCaseBase(final String fileName) {
		try{
			/* Read each case from the file and store in memory */
			final BufferedReader br = new BufferedReader(new FileReader(fileName));
			String line = "";
			
			while((line = br.readLine()) != null){
				// Each line represents 1 case, made up of one or more features plus one solution
				final Case c1 = new Case();
				final String[] features = line.split("/");
				
				if(features.length > 1){
					// Each feature has the following format: FeatureName:Value:Indexed
					for(final String f : features){
						final String[] details = f.split(":");
						
						// If there are three elements it is a feature, otherwise it must be a solution.
						if(details.length == NUMBER_DETAILS){
							c1.addFeature(new Feature(details[0], new Value(details[1]), "true".equalsIgnoreCase(details[2]) ? true : false));
						} else {
							final int index = details[0].indexOf('|');
							c1.setSolution(new Solution(details[0].substring(0, index)));
							c1.setOutcomeValue(Double.parseDouble(details[0].substring(index+1)));
						}
					}
					caseBase.add(c1);
				}
			}
			br.close();
		} catch (FileNotFoundException e) {
			System.out.println("The file: " + fileName + " was not found.");
		} catch (IOException e) {
			System.out.println(e);
		}
	}
	
	/**
	 * Retain the target case by adding its features and solution to the flat file used to represent the case-base.
	 * @param targetCase the case to be retained
	 * @param fileName the file to retain the persisted case
	 */
	public void retain(final Case targetCase, final String fileName){
		// First make the case suitable for saving in the case base
		targetCase.convertTargetCaseToSourceCase();
		// Update the case base with new case
		caseBase.add(targetCase);
		
		// Update the file used to persist cases
		try{
			final BufferedWriter bw = new BufferedWriter(new FileWriter(fileName, true));
			bw.write("\n" + targetCase.getFlatFileFormat());
			bw.flush();
			bw.close();
		} catch (FileNotFoundException e) {
			System.out.println("The file: " + fileName + " was not found.");
		} catch (IOException e) {
			System.out.println(e);
		}
	}
	
	/**
	 * Retains an existing case in the case base to reflect changes made to the outcome of the case.
	 * @param existingCase the case whose outcome has been modified
	 * @param fileName the file to retain the persisted case 
	 */
	public void retainWithReplacement(final Case existingCase, final String fileName){
		try{
			final File inFile = new File(fileName);
			// Match the features and the solution, but not the outcome as this is the value that is being modified.
			final String matchingCaseInfo = existingCase.getFlatFileFormat().substring(0, existingCase.getFlatFileFormat().indexOf('|'));
			
			if (!inFile.isFile()) {
				System.out.println("Parameter is not an existing file");
				return;
			}
			
			// Construct the new file that will later be renamed to the original filename. 
			final File tempFile = new File(inFile.getAbsolutePath() + ".tmp");
		      
			final BufferedReader br = new BufferedReader(new FileReader(inFile));
			final BufferedWriter bw = new BufferedWriter(new FileWriter(tempFile));
		      
			String line = null;

			// Read from the original file and write to the new 
			// unless content matches data to be removed.
			while ((line = br.readLine()) != null) {
				if(line.trim().contains(matchingCaseInfo)){
					bw.write(existingCase.getFlatFileFormat());
					bw.write("\n");
				} else if(!"".equals(line.trim())){
					bw.write(line);
					bw.write("\n");
				}
				bw.flush();
			}
			bw.close();
			br.close();
			
			//Delete the original file
			if (!inFile.delete()) {
				System.out.println("Could not delete file");
				return;
			} 

			//Rename the new file to the filename the original file had.
			if (!tempFile.renameTo(inFile)){
				System.out.println("Could not rename file");
			}
		} catch (FileNotFoundException e) {
			System.out.println("The file: " + fileName + " was not found.");
		} catch (IOException e) {
			System.out.println(e);
		}
	}
	
	/**
	 * Provides a reference to the case-base.
	 * @return the case base
	 */
	public List<Case> getAllCases(){
		return caseBase;
	}
	
	/**
	 * Prints all the cases in the current case-base to standard output.
	 */
	public void printAllCases(){
		for(final Case c : caseBase){
			System.out.println(c);
		}
	}
}
