/*
 * Copyright 2009 Jonathan Rubin
 * This file is part of Camus.
 *
 * Camus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Camus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Camus.  If not, see <http://www.gnu.org/licenses/>.
 */

package cbr.framework;

import java.util.ArrayList;
import java.util.Collections;
import java.util.List;

/**
 * <p><code>Reasoner</code> provides a collection of static, utility methods to access a case base
 * and perform reasoning tasks such as comparing a target case to source cases.</p> 
 *
 */
public class Reasoner {

	/**
	 * Given a novel scenario represented as a case find the <code>N</code> most similar cases in the <code>caseBase</code> provided.
	 * @param n the number of similar cases to return
	 * @param targetCase the novel scenario
	 * @param caseBase the collection of cases to test against
	 * @return the n most similar cases as a list
	 */
	public static List<Case> getNMostSimilarCases(final int n, final Case targetCase, final CaseBase caseBase){
		final List<Case> cases = caseBase.getAllCases(); 
		if(n > cases.size()){
			throw new IllegalArgumentException("Cannot retrieve " + n + " cases as there are only " + cases.size() + " cases in the case base.");
		}
		
		// Cycle through all cases in the case base and compute the similarity to the target case
		for(final Case c : cases){
			c.calculateSimilarity(targetCase);
		}
		Collections.sort(cases);

		// Create a new auxiliary list to store the top n cases
		final ArrayList<Case> similarCases = new ArrayList<Case>(n);
		
		int count = 0;
		for(final Case c : cases){
			if(count++ < n){
				similarCases.add(c);
			} else {
				break;
			}
		}
		return similarCases;
	}
	
	/**
	 * Retrieves the <code>case</code> from the case base that most resembles the target case. 
	 * @param targetCase the novel case
	 * @param caseBase the source cases
	 * @return the most similar source case
	 */
	public static Case getMostSimilarCase(final Case targetCase, final CaseBase caseBase){
		return getNMostSimilarCases(1, targetCase, caseBase).get(0);
	}
	
	/**
	 * Retrieves the <code>cases</code> from the case base that are over (or equal to) a specified similarity threshold, 
	 * when compared to the <code>targetCase</code>.  
	 * @param threshold the similarity threshold
	 * @param targetCase the novel case
	 * @param caseBase the source cases
	 * @return the cases that are greater than or equal to the similarity threshold
	 */
	public static List<Case> getCasesOverSimilarityThreshold(double threshold, final Case targetCase, final CaseBase caseBase){
		final List<Case> cases = caseBase.getAllCases();
		final ArrayList<Case> similarCases = new ArrayList<Case>();
		
		// Find all the cases within the case base that are over the similarity threshold specified
		for(final Case c : cases){
			c.calculateSimilarity(targetCase);
			if(c.getSimilarity() >= threshold ){
				similarCases.add(c);
			}
		}
		
		return similarCases;
	}
}
