/*
 * Copyright 2009 Jonathan Rubin
 * This file is part of Camus.
 *
 * Camus is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * Camus is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Camus.  If not, see <http://www.gnu.org/licenses/>.
 */

package cbr.framework;

/**
 * This class represents a target case feature. It differs from a normal feature in that a similarity metric and weight is
 * specified to be used during the retrieval process. It can only be used with TargetCases.
 *
 */
public class TargetFeature extends Feature {
	private SimilarityMetric similarityMetric;
	private double weight;
	
	/**
	 * Create a feature for a target case.
	 * @param name the name of the feature
	 * @param value the feature's value
	 * @param sm the similarity metric to use
	 * @param weight the feature's weight
	 */
	public TargetFeature(final String name, final Value value, final SimilarityMetric sm, final double weight){
		super(name, value, weight > 0.0 ? true : false);
		similarityMetric = sm;
		this.weight = weight;
	}
	
	/**
	 * Get the <code>SimilarityMetric</code> to be used for this feature. 
	 * @return the <code>SimilarityMetric</code>
	 */
	public SimilarityMetric getSimilarityMetric() {
		return similarityMetric;
	}

	/**
	 * Set the <code>SimilarityMetric</code> to be used for this feature.
	 * @param similarityMetric the <code>SimilarityMetric</code>
	 */
	public void setSimilarityMetric(SimilarityMetric similarityMetric) {
		if(similarityMetric == null){
			return;
		}
		this.similarityMetric = similarityMetric;
	}

	/**
	 * Get the feature's weighting.
	 * @return the weight value
	 */
	public double getWeight() {
		return weight;
	}

	/**
	 * Set the feature's weighting.
	 * @param weight the weight value
	 */
	public void setWeight(double weight) {
		this.weight = weight;
	}
}
